/*
 * Copyright (c) 2005 (Mike) Maurice Kienenberger (mkienenb@alaska.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.gamenet.swing.controls;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.ListCellRenderer;

public class StringComboBox extends JComboBox implements ActionListener
{
    private NumberLabelValueHolder[] valueLabelArray;
    private IntValueHolder intValueHolder;
    
    public StringComboBox(NumberLabelValueHolder srcValueLabelArray[], IntValueHolder srcIntValueHolder)
    {
        super(srcValueLabelArray);
        this.valueLabelArray = srcValueLabelArray;
        this.intValueHolder = srcIntValueHolder;
        
        this.setSelectedItem(NumberLabelValueHolder.getLabelForNumberInArray(intValueHolder.getValue(), valueLabelArray));
         
        ListCellRenderer renderer = new AbstractStringListCellRenderer()
        {
            protected String getStringForValue(Object value)
            {
                if (null == value)  return  "Error: empty value";	
                return ((NumberLabelValueHolder)value).getLabel();
            }
        };
        this.setRenderer(renderer);
        this.addActionListener(this);
    }

	public void actionPerformed(ActionEvent e)
	{
        JComboBox cb = (JComboBox)e.getSource();
        NumberLabelValueHolder item = (NumberLabelValueHolder)cb.getSelectedItem();
        intValueHolder.setValue(item.getNumber());
    }

    public NumberLabelValueHolder[] getNumberLabelValueHolder()
    {
        return valueLabelArray;
    }

    public IntValueHolder getIntValueHolder()
    {
        return intValueHolder;
    }
}
